// AUTO-GENERATED BY UpdateGenVersion.bat e50b558106d8eba96457909d9f0c2f12b2898d7a
const char GenVersion_Version[] = "0.4.4-dev";
const char GenVersion_Commit[] = "e50b558106d8eba96457909d9f0c2f12b2898d7a";
